/*
   Copyright (c) 2018 LinkedIn Corp.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.linkedin.entitystream;

/**
 * Reader is an {@link Observer} that has the extended ability to read data
 * and drive and control the data flow of an {@link EntityStream}
 *
 * It is expected that the methods {@link #onInit}, {@link #onDataAvailable}, {@link #onDone} and {@link #onError} are
 * invoked in a thread-safe manner, usually by EntityStream.
 *
 * @author Zhenkai Zhu
 */
public interface Reader<T> extends Observer<T>
{
  /**
   * This is called when the reader is set to the EntityStream
   *
   * @param rh the ReadHandle {@link ReadHandle} provided to this reader.
   */
  void onInit(ReadHandle rh);

  /**
   * {@inheritDoc}
   *
   * Unlike {@link Observer}, a Reader will not receive any {@link AbortedException} in this method when it
   * cancels reading.
   *
   * @param e the cause of the error.
   */
  @Override
  void onError(Throwable e);

}
